@@ -11,15 +11,16 @@ import warnings
 import click
 import dask
 from dask.system import CPU_COUNT
-from distributed import Nanny, Security
+from distributed import Nanny
 from distributed.cli.utils import check_python_3, install_signal_handlers
 from distributed.comm import get_address_host_port
+from distributed.deploy.utils import nprocesses_nthreads
 from distributed.preloading import validate_preload_argv
 from distributed.proctitle import (
     enable_proctitle_on_children,
     enable_proctitle_on_current,
 )
-from distributed.utils import deserialize_for_cli, import_term
+from distributed.utils import import_term
 
 from tlz import valmap
 from tornado.ioloop import IOLoop, TimeoutError
@@ -124,10 +125,12 @@ pem_file_option_type = click.Path(exists=True, resolve_path=True)
 @click.option("--nthreads", type=int, default=0, help="Number of threads per process.")
 @click.option(
     "--nprocs",
-    type=int,
+    type=str,
     default=1,
     show_default=True,
-    help="Number of worker processes to launch.",
+    help="Number of worker processes to launch. "
+    "If negative, then (CPU_COUNT + 1 + nprocs) is used. "
+    "Set to 'auto' to set nprocs and nthreads dynamically based on CPU_COUNT",
 )
 @click.option(
     "--name",
@@ -141,11 +144,14 @@ pem_file_option_type = click.Path(exists=True, resolve_path=True)
     "--memory-limit",
     default="auto",
     show_default=True,
-    help="Bytes of memory per process that the worker can use. "
-    "This can be an integer (bytes), "
-    "float (fraction of total system memory), "
-    "string (like 5GB or 5000M), "
-    "'auto', or zero for no memory management",
+    help="""\b
+    Bytes of memory per process that the worker can use.
+    This can be:
+    - an integer (bytes), note 0 is a special case for no memory management.
+    - a float (fraction of total system memory).
+    - a string (like 5GB or 5000M).
+    - 'auto' for automatically computing the memory limit.
+    """,
 )
 @click.option(
     "--reconnect/--no-reconnect",
@@ -278,17 +284,29 @@ def main(
         )
         dashboard = bokeh
 
-    sec = Security(
-        **{
-            k: v
-            for k, v in [
-                ("tls_ca_file", tls_ca_file),
-                ("tls_worker_cert", tls_cert),
-                ("tls_worker_key", tls_key),
-            ]
-            if v is not None
-        }
-    )
+    sec = {
+        k: v
+        for k, v in [
+            ("tls_ca_file", tls_ca_file),
+            ("tls_worker_cert", tls_cert),
+            ("tls_worker_key", tls_key),
+        ]
+        if v is not None
+    }
+
+    if nprocs == "auto":
+        nprocs, nthreads = nprocesses_nthreads()
+    else:
+        nprocs = int(nprocs)
+
+    if nprocs < 0:
+        nprocs = CPU_COUNT + 1 + nprocs
+
+    if nprocs <= 0:
+        logger.error(
+            "Failed to launch worker. Must specify --nprocs so that there's at least one process."
+        )
+        sys.exit(1)
 
     if nprocs > 1 and not nanny:
         logger.error(
@@ -384,11 +402,6 @@ def main(
     with suppress(TypeError, ValueError):
         name = int(name)
 
-    if "DASK_INTERNAL_INHERIT_CONFIG" in os.environ:
-        config = deserialize_for_cli(os.environ["DASK_INTERNAL_INHERIT_CONFIG"])
-        # Update the global config given priority to the existing global config
-        dask.config.update(dask.config.global_config, config, priority="old")
-
     nannies = [
         t(
             scheduler,
