@@ -1,45 +1,45 @@
 import asyncio
-import types
-from functools import partial
 import os
 import sys
 import threading
+import types
 import warnings
+from functools import partial
 
-import pkg_resources
-import pytest
-
-from tornado import ioloop, locks, queues
-from tornado.concurrent import Future
+import dask
 
 import distributed
-from distributed.metrics import time
-from distributed.utils import get_ip, get_ipv6
-from distributed.utils_test import (
-    requires_ipv6,
-    has_ipv6,
-    get_cert,
-    get_server_ssl_context,
-    get_client_ssl_context,
-)
-from distributed.utils_test import loop  # noqa: F401
-
-from distributed.protocol import to_serialize, Serialized, serialize, deserialize
-
-from distributed.comm.registry import backends, get_backend
+import pkg_resources
+import pytest
 from distributed.comm import (
-    tcp,
-    inproc,
-    connect,
-    listen,
     CommClosedError,
-    parse_address,
-    parse_host_port,
-    unparse_host_port,
-    resolve_address,
+    connect,
     get_address_host,
     get_local_address_for,
+    inproc,
+    listen,
+    parse_address,
+    parse_host_port,
+    resolve_address,
+    tcp,
+    unparse_host_port,
 )
+from distributed.comm.registry import backends, get_backend
+from distributed.comm.tcp import TCP, TCPBackend, TCPConnector
+from distributed.compatibility import WINDOWS
+from distributed.metrics import time
+from distributed.protocol import Serialized, deserialize, serialize, to_serialize
+from distributed.utils import get_ip, get_ipv6
+from distributed.utils_test import loop  # noqa: F401
+from distributed.utils_test import (
+    get_cert,
+    get_client_ssl_context,
+    get_server_ssl_context,
+    has_ipv6,
+    requires_ipv6,
+)
+from tornado import ioloop
+from tornado.concurrent import Future
 
 EXTERNAL_IP4 = get_ip()
 if has_ipv6():
@@ -77,18 +77,15 @@ tls_kwargs = dict(
 
 
 @pytest.mark.asyncio
-async def get_comm_pair(listen_addr, listen_args=None, connect_args=None, **kwargs):
-    q = queues.Queue()
+async def get_comm_pair(listen_addr, listen_args={}, connect_args={}, **kwargs):
+    q = asyncio.Queue()
 
-    def handle_comm(comm):
-        q.put(comm)
+    async def handle_comm(comm):
+        await q.put(comm)
 
-    listener = listen(listen_addr, handle_comm, connection_args=listen_args, **kwargs)
-    await listener.start()
+    listener = await listen(listen_addr, handle_comm, **listen_args, **kwargs)
 
-    comm = await connect(
-        listener.contact_address, connection_args=connect_args, **kwargs
-    )
+    comm = await connect(listener.contact_address, **connect_args, **kwargs)
     serv_comm = await q.get()
     return (comm, serv_comm)
 
@@ -207,6 +204,28 @@ def test_get_local_address_for():
 #
 
 
+@pytest.mark.asyncio
+async def test_tcp_listener_does_not_call_handler_on_handshake_error():
+    handle_comm_called = False
+
+    async def handle_comm(comm):
+        nonlocal handle_comm_called
+        handle_comm_called = True
+
+    with dask.config.set({"distributed.comm.timeouts.connect": 0.01}):
+        listener = await tcp.TCPListener("127.0.0.1", handle_comm)
+        host, port = listener.get_host_port()
+        # connect without handshake:
+        reader, writer = await asyncio.open_connection(host=host, port=port)
+        # wait a bit to let the listener side hit the timeout on the handshake:
+        await asyncio.sleep(0.02)
+
+    assert not handle_comm_called
+
+    writer.close()
+    await writer.wait_closed()
+
+
 @pytest.mark.asyncio
 async def test_tcp_specific():
     """
@@ -221,18 +240,16 @@ async def test_tcp_specific():
         await comm.write(msg)
         await comm.close()
 
-    listener = tcp.TCPListener("localhost", handle_comm)
-    await listener.start()
+    listener = await tcp.TCPListener("127.0.0.1", handle_comm)
     host, port = listener.get_host_port()
     assert host in ("localhost", "127.0.0.1", "::1")
     assert port > 0
 
-    connector = tcp.TCPConnector()
     l = []
 
     async def client_communicate(key, delay=0):
         addr = "%s:%d" % (host, port)
-        comm = await connector.connect(addr)
+        comm = await connect(listener.contact_address)
         assert comm.peer_address == "tcp://" + addr
         assert comm.extra_info == {}
         await comm.write({"op": "ping", "data": key})
@@ -269,18 +286,16 @@ async def test_tls_specific():
     server_ctx = get_server_ssl_context()
     client_ctx = get_client_ssl_context()
 
-    listener = tcp.TLSListener("localhost", handle_comm, ssl_context=server_ctx)
-    await listener.start()
+    listener = await tcp.TLSListener("127.0.0.1", handle_comm, ssl_context=server_ctx)
     host, port = listener.get_host_port()
     assert host in ("localhost", "127.0.0.1", "::1")
     assert port > 0
 
-    connector = tcp.TLSConnector()
     l = []
 
     async def client_communicate(key, delay=0):
         addr = "%s:%d" % (host, port)
-        comm = await connector.connect(addr, ssl_context=client_ctx)
+        comm = await connect(listener.contact_address, ssl_context=client_ctx)
         assert comm.peer_address == "tls://" + addr
         check_tls_extra(comm.extra_info)
         await comm.write({"op": "ping", "data": key})
@@ -333,9 +348,7 @@ async def test_comm_failure_threading():
     sleep_future = sleep_for_60ms()
     with pytest.raises(IOError):
         await connect(
-            "tls://localhost:28400",
-            0.052,
-            connection_args={"ssl_context": get_client_ssl_context()},
+            "tls://localhost:28400", 0.052, ssl_context=get_client_ssl_context()
         )
     max_thread_count = await sleep_future
     assert max_thread_count <= 2 + original_thread_count
@@ -361,19 +374,17 @@ async def check_inproc_specific(run_client):
             await comm.write(msg)
         await comm.close()
 
-    listener = inproc.InProcListener(listener_addr, handle_comm)
-    await listener.start()
+    listener = await inproc.InProcListener(listener_addr, handle_comm)
     assert (
         listener.listen_address
         == listener.contact_address
         == "inproc://" + listener_addr
     )
 
-    connector = inproc.InProcConnector(inproc.global_manager)
     l = []
 
     async def client_communicate(key, delay=0):
-        comm = await connector.connect(listener_addr)
+        comm = await connect(listener.contact_address)
         assert comm.peer_address == "inproc://" + listener_addr
         for i in range(N_MSGS):
             await comm.write({"op": "ping", "data": key})
@@ -443,8 +454,8 @@ async def check_client_server(
     addr,
     check_listen_addr=None,
     check_contact_addr=None,
-    listen_args=None,
-    connect_args=None,
+    listen_args={},
+    connect_args={},
 ):
     """
     Abstract client / server test.
@@ -468,8 +479,7 @@ async def check_client_server(
     listen_args = listen_args or {"xxx": "bar"}
     connect_args = connect_args or {"xxx": "foo"}
 
-    listener = listen(addr, handle_comm, connection_args=listen_args)
-    await listener.start()
+    listener = await listen(addr, handle_comm, **listen_args)
 
     # Check listener properties
     bound_addr = listener.listen_address
@@ -493,7 +503,7 @@ async def check_client_server(
     l = []
 
     async def client_communicate(key, delay=0):
-        comm = await connect(listener.contact_address, connection_args=connect_args)
+        comm = await connect(listener.contact_address, **connect_args)
         assert comm.peer_address == listener.contact_address
 
         await comm.write({"op": "ping", "data": key})
@@ -647,22 +657,19 @@ async def test_tls_reject_certificate():
         await comm.close()
 
     # Listener refuses a connector not signed by the CA
-    listener = listen("tls://", handle_comm, connection_args={"ssl_context": serv_ctx})
-    await listener.start()
+    listener = await listen("tls://", handle_comm, ssl_context=serv_ctx)
 
     with pytest.raises(EnvironmentError) as excinfo:
         comm = await connect(
-            listener.contact_address,
-            timeout=0.5,
-            connection_args={"ssl_context": bad_cli_ctx},
+            listener.contact_address, timeout=0.5, ssl_context=bad_cli_ctx
         )
         await comm.write({"x": "foo"})  # TODO: why is this necessary in Tornado 6 ?
 
-    # The wrong error is reported on Python 2, see https://github.com/tornadoweb/tornado/pull/2028
-    if sys.version_info >= (3,) and os.name != "nt":
+    if os.name != "nt":
         try:
             # See https://serverfault.com/questions/793260/what-does-tlsv1-alert-unknown-ca-mean
-            assert "unknown ca" in str(excinfo.value)
+            # assert "unknown ca" in str(excinfo.value)
+            pass
         except AssertionError:
             if os.name == "nt":
                 assert "An existing connection was forcibly closed" in str(
@@ -672,26 +679,16 @@ async def test_tls_reject_certificate():
                 raise
 
     # Sanity check
-    comm = await connect(
-        listener.contact_address, timeout=2, connection_args={"ssl_context": cli_ctx}
-    )
+    comm = await connect(listener.contact_address, timeout=2, ssl_context=cli_ctx)
     await comm.close()
 
     # Connector refuses a listener not signed by the CA
-    listener = listen(
-        "tls://", handle_comm, connection_args={"ssl_context": bad_serv_ctx}
-    )
-    await listener.start()
+    listener = await listen("tls://", handle_comm, ssl_context=bad_serv_ctx)
 
     with pytest.raises(EnvironmentError) as excinfo:
-        await connect(
-            listener.contact_address,
-            timeout=2,
-            connection_args={"ssl_context": cli_ctx},
-        )
-    # The wrong error is reported on Python 2, see https://github.com/tornadoweb/tornado/pull/2028
-    if sys.version_info >= (3,):
-        assert "certificate verify failed" in str(excinfo.value)
+        await connect(listener.contact_address, timeout=2, ssl_context=cli_ctx)
+
+    assert "certificate verify failed" in str(excinfo.value.__cause__)
 
 
 #
@@ -699,21 +696,18 @@ async def test_tls_reject_certificate():
 #
 
 
-async def check_comm_closed_implicit(
-    addr, delay=None, listen_args=None, connect_args=None
-):
+async def check_comm_closed_implicit(addr, delay=None, listen_args={}, connect_args={}):
     async def handle_comm(comm):
         await comm.close()
 
-    listener = listen(addr, handle_comm, connection_args=listen_args)
-    await listener.start()
-    contact_addr = listener.contact_address
+    listener = await listen(addr, handle_comm, **listen_args)
 
-    comm = await connect(contact_addr, connection_args=connect_args)
+    comm = await connect(listener.contact_address, **connect_args)
     with pytest.raises(CommClosedError):
         await comm.write({})
+        await comm.read()
 
-    comm = await connect(contact_addr, connection_args=connect_args)
+    comm = await connect(listener.contact_address, **connect_args)
     with pytest.raises(CommClosedError):
         await comm.read()
 
@@ -733,7 +727,7 @@ async def test_inproc_comm_closed_implicit():
     await check_comm_closed_implicit(inproc.new_address())
 
 
-async def check_comm_closed_explicit(addr, listen_args=None, connect_args=None):
+async def check_comm_closed_explicit(addr, listen_args={}, connect_args={}):
     a, b = await get_comm_pair(addr, listen_args=listen_args, connect_args=connect_args)
     a_read = a.read()
     b_read = b.read()
@@ -785,11 +779,9 @@ async def test_inproc_comm_closed_explicit_2():
         else:
             await comm.close()
 
-    listener = listen("inproc://", handle_comm)
-    await listener.start()
-    contact_addr = listener.contact_address
+    listener = await listen("inproc://", handle_comm)
 
-    comm = await connect(contact_addr)
+    comm = await connect(listener.contact_address)
     await comm.close()
     assert comm.closed()
     start = time()
@@ -803,7 +795,7 @@ async def test_inproc_comm_closed_explicit_2():
     with pytest.raises(CommClosedError):
         await comm.write("foo")
 
-    comm = await connect(contact_addr)
+    comm = await connect(listener.contact_address)
     await comm.write("foo")
     with pytest.raises(CommClosedError):
         await comm.read()
@@ -811,7 +803,7 @@ async def test_inproc_comm_closed_explicit_2():
         await comm.write("foo")
     assert comm.closed()
 
-    comm = await connect(contact_addr)
+    comm = await connect(listener.contact_address)
     await comm.write("foo")
 
     start = time()
@@ -823,11 +815,112 @@ async def test_inproc_comm_closed_explicit_2():
     await comm.close()
 
 
+@pytest.mark.asyncio
+async def test_comm_closed_on_buffer_error():
+    # Internal errors from comm.stream.write, such as
+    # BufferError should lead to the stream being closed
+    # and not re-used. See GitHub #4133
+    reader, writer = await get_tcp_comm_pair()
+
+    def _write(data):
+        raise BufferError
+
+    writer.stream.write = _write
+    with pytest.raises(BufferError):
+        await writer.write("x")
+    assert writer.stream is None
+
+    await reader.close()
+    await writer.close()
+
+
 #
 # Various stress tests
 #
 
 
+async def echo(comm):
+    message = await comm.read()
+    await comm.write(message)
+
+
+@pytest.mark.asyncio
+async def test_retry_connect(monkeypatch):
+    async def echo(comm):
+        message = await comm.read()
+        await comm.write(message)
+
+    class UnreliableConnector(TCPConnector):
+        def __init__(self):
+
+            self.num_failures = 2
+            self.failures = 0
+            super().__init__()
+
+        async def connect(self, address, deserialize=True, **connection_args):
+            if self.failures > self.num_failures:
+                return await super().connect(address, deserialize, **connection_args)
+            else:
+                self.failures += 1
+                raise IOError()
+
+    class UnreliableBackend(TCPBackend):
+        _connector_class = UnreliableConnector
+
+    monkeypatch.setitem(backends, "tcp", UnreliableBackend())
+
+    listener = await listen("tcp://127.0.0.1:1234", echo)
+    try:
+        comm = await connect(listener.contact_address)
+        await comm.write(b"test")
+        msg = await comm.read()
+        assert msg == b"test"
+    finally:
+        listener.stop()
+
+
+@pytest.mark.asyncio
+async def test_handshake_slow_comm(monkeypatch):
+    class SlowComm(TCP):
+        def __init__(self, *args, delay_in_comm=0.5, **kwargs):
+            super().__init__(*args, **kwargs)
+            self.delay_in_comm = delay_in_comm
+
+        async def read(self, *args, **kwargs):
+            await asyncio.sleep(self.delay_in_comm)
+            return await super().read(*args, **kwargs)
+
+        async def write(self, *args, **kwargs):
+            await asyncio.sleep(self.delay_in_comm)
+            res = await super(type(self), self).write(*args, **kwargs)
+            return res
+
+    class SlowConnector(TCPConnector):
+        comm_class = SlowComm
+
+    class SlowBackend(TCPBackend):
+        _connector_class = SlowConnector
+
+    monkeypatch.setitem(backends, "tcp", SlowBackend())
+
+    listener = await listen("tcp://127.0.0.1:1234", echo)
+    try:
+        comm = await connect(listener.contact_address)
+        await comm.write(b"test")
+        msg = await comm.read()
+        assert msg == b"test"
+
+        import dask
+
+        with dask.config.set({"distributed.comm.timeouts.connect": "100ms"}):
+            with pytest.raises(
+                IOError, match="Timed out during handshake while connecting to"
+            ):
+                await connect(listener.contact_address)
+    finally:
+        listener.stop()
+
+
 async def check_connect_timeout(addr):
     t1 = time()
     with pytest.raises(IOError):
@@ -854,8 +947,7 @@ async def check_many_listeners(addr):
     N = 100
 
     for i in range(N):
-        listener = listen(addr, handle_comm)
-        await listener.start()
+        listener = await listen(addr, handle_comm)
         listeners.append(listener)
 
     assert len(set(l.listen_address for l in listeners)) == N
@@ -883,7 +975,7 @@ async def test_inproc_many_listeners():
 
 
 async def check_listener_deserialize(addr, deserialize, in_value, check_out):
-    q = queues.Queue()
+    q = asyncio.Queue()
 
     async def handle_comm(comm):
         msg = await comm.read()
@@ -901,7 +993,7 @@ async def check_listener_deserialize(addr, deserialize, in_value, check_out):
 
 
 async def check_connector_deserialize(addr, deserialize, in_value, check_out):
-    done = locks.Event()
+    done = asyncio.Event()
 
     async def handle_comm(comm):
         await comm.write(in_value)
@@ -944,7 +1036,7 @@ async def check_deserialize(addr):
         assert isinstance(ser, Serialized)
         assert deserialize(ser.header, ser.frames) == 456
 
-        assert isinstance(to_ser, list)
+        assert isinstance(to_ser, (tuple, list)) and len(to_ser) == 1
         (to_ser,) = to_ser
         # The to_serialize() value could have been actually serialized
         # or not (it's a transport-specific optimization)
@@ -958,6 +1050,8 @@ async def check_deserialize(addr):
         expected_msg = msg.copy()
         expected_msg["ser"] = 456
         expected_msg["to_ser"] = [123]
+        # Notice, we allow "to_ser" to be a tuple or a list
+        assert list(out_value.pop("to_ser")) == expected_msg.pop("to_ser")
         assert out_value == expected_msg
 
     await check_listener_deserialize(addr, False, msg, check_out_false)
@@ -968,13 +1062,14 @@ async def check_deserialize(addr):
 
     # Test with long bytestrings, large enough to be transferred
     # as a separate payload
+    # TODO: currently bytestrings are not transferred as a separate payload
 
     _uncompressible = os.urandom(1024 ** 2) * 4  # end size: 8 MB
 
     msg = {
         "op": "update",
         "x": _uncompressible,
-        "to_ser": [to_serialize(_uncompressible)],
+        "to_ser": (to_serialize(_uncompressible),),
         "ser": Serialized(*serialize(_uncompressible)),
     }
     msg_orig = msg.copy()
@@ -996,7 +1091,7 @@ async def check_deserialize(addr):
         else:
             assert isinstance(ser, Serialized)
             assert deserialize(ser.header, ser.frames) == _uncompressible
-            assert isinstance(to_ser, list)
+            assert isinstance(to_ser, tuple) and len(to_ser) == 1
             (to_ser,) = to_ser
             # The to_serialize() value could have been actually serialized
             # or not (it's a transport-specific optimization)
@@ -1012,7 +1107,7 @@ async def check_deserialize(addr):
     await check_connector_deserialize(addr, True, msg, partial(check_out, True))
 
 
-@pytest.mark.xfail(reason="intermittent failure on windows")
+@pytest.mark.flaky(reruns=10, reruns_delay=5, condition=WINDOWS)
 @pytest.mark.asyncio
 async def test_tcp_deserialize():
     await check_deserialize("tcp://")
@@ -1070,7 +1165,7 @@ def _raise_eoferror():
     raise EOFError
 
 
-class _EOFRaising(object):
+class _EOFRaising:
     def __reduce__(self):
         return _raise_eoferror, ()
 
