@@ -35,6 +35,8 @@ logger = logging.getLogger(__name__)
 ucp = None
 host_array = None
 device_array = None
+ucx_create_endpoint = None
+ucx_create_listener = None
 
 
 def synchronize_stream(stream=0):
@@ -47,7 +49,7 @@ def synchronize_stream(stream=0):
 
 
 def init_once():
-    global ucp, host_array, device_array
+    global ucp, host_array, device_array, ucx_create_endpoint, ucx_create_listener
     if ucp is not None:
         return
 
@@ -107,6 +109,19 @@ def init_once():
             pool_allocator=True, managed_memory=False, initial_pool_size=pool_size
         )
 
+    try:
+        from ucp.endpoint_reuse import EndpointReuse
+    except ImportError:
+        ucx_create_endpoint = ucp.create_endpoint
+        ucx_create_listener = ucp.create_listener
+    else:
+        if dask.config.get("ucx.reuse-endpoints"):
+            ucx_create_endpoint = EndpointReuse.create_endpoint
+            ucx_create_listener = EndpointReuse.create_listener
+        else:
+            ucx_create_endpoint = ucp.create_endpoint
+            ucx_create_listener = ucp.create_listener
+
 
 class UCX(Comm):
     """Comm object using UCP.
@@ -136,7 +151,7 @@ class UCX(Comm):
 
     The expected read cycle is
 
-    1. Read the frame describing number of frames
+    1. Read the frame describing if connection is closing and number of frames
     2. Read the frame describing whether each data frame is gpu-bound
     3. Read the frame describing whether each data frame is sized
     4. Read all the data frames.
@@ -176,23 +191,28 @@ class UCX(Comm):
                     serializers = ("cuda", "dask", "pickle", "error")
                 # msg can also be a list of dicts when sending batched messages
                 frames = await to_frames(
-                    msg, serializers=serializers, on_error=on_error
+                    msg,
+                    serializers=serializers,
+                    on_error=on_error,
+                    allow_offload=self.allow_offload,
                 )
                 nframes = len(frames)
                 cuda_frames = tuple(
                     hasattr(f, "__cuda_array_interface__") for f in frames
                 )
                 sizes = tuple(nbytes(f) for f in frames)
-                send_frames = [
-                    each_frame
-                    for each_frame, each_size in zip(frames, sizes)
-                    if each_size
-                ]
+                cuda_send_frames, send_frames = zip(
+                    *(
+                        (is_cuda, each_frame)
+                        for is_cuda, each_frame in zip(cuda_frames, frames)
+                        if nbytes(each_frame) > 0
+                    )
+                )
 
                 # Send meta data
 
-                # Send # of frames (uint64)
-                await self.ep.send(struct.pack("Q", nframes))
+                # Send close flag and number of frames (_Bool, int64)
+                await self.ep.send(struct.pack("?Q", False, nframes))
                 # Send which frames are CUDA (bool) and
                 # how large each frame is (uint64)
                 await self.ep.send(
@@ -206,7 +226,7 @@ class UCX(Comm):
                 #  syncing the default stream will wait for other non-blocking CUDA streams.
                 # Note this is only sufficient if the memory being sent is not currently in use on
                 # non-blocking CUDA streams.
-                if any(cuda_frames):
+                if any(cuda_send_frames):
                     synchronize_stream(0)
 
                 for each_frame in send_frames:
@@ -227,11 +247,13 @@ class UCX(Comm):
             try:
                 # Recv meta data
 
-                # Recv # of frames (uint64)
-                nframes_fmt = "Q"
-                nframes = host_array(struct.calcsize(nframes_fmt))
-                await self.ep.recv(nframes)
-                (nframes,) = struct.unpack(nframes_fmt, nframes)
+                # Recv close flag and number of frames (_Bool, int64)
+                msg = host_array(struct.calcsize("?Q"))
+                await self.ep.recv(msg)
+                (shutdown, nframes) = struct.unpack("?Q", msg)
+
+                if shutdown:  # The writer is closing the connection
+                    raise CancelledError("Connection closed by writer")
 
                 # Recv which frames are CUDA (bool) and
                 # how large each frame is (uint64)
@@ -249,25 +271,39 @@ class UCX(Comm):
                     device_array(each_size) if is_cuda else host_array(each_size)
                     for is_cuda, each_size in zip(cuda_frames, sizes)
                 ]
-                recv_frames = [
-                    each_frame for each_frame in frames if len(each_frame) > 0
-                ]
+                cuda_recv_frames, recv_frames = zip(
+                    *(
+                        (is_cuda, each_frame)
+                        for is_cuda, each_frame in zip(cuda_frames, frames)
+                        if nbytes(each_frame) > 0
+                    )
+                )
 
                 # It is necessary to first populate `frames` with CUDA arrays and synchronize
                 # the default stream before starting receiving to ensure buffers have been allocated
-                if any(cuda_frames):
+                if any(cuda_recv_frames):
                     synchronize_stream(0)
 
                 for each_frame in recv_frames:
                     await self.ep.recv(each_frame)
                 msg = await from_frames(
-                    frames, deserialize=self.deserialize, deserializers=deserializers
+                    frames,
+                    deserialize=self.deserialize,
+                    deserializers=deserializers,
+                    allow_offload=self.allow_offload,
                 )
                 return msg
 
     async def close(self):
         if self._ep is not None:
-            await self._ep.close()
+            try:
+                await self.ep.send(struct.pack("?Q", True, 0))
+            except ucp.exceptions.UCXError:
+                # If the other end is in the process of closing,
+                # UCX will sometimes raise a `Input/output` error,
+                # which we can ignore.
+                pass
+            self.abort()
             self._ep = None
 
     def abort(self):
@@ -295,7 +331,7 @@ class UCXConnector(Connector):
         logger.debug("UCXConnector.connect: %s", address)
         ip, port = parse_host_port(address)
         init_once()
-        ep = await ucp.create_endpoint(ip, port)
+        ep = await ucx_create_endpoint(ip, port)
         return self.comm_class(
             ep,
             local_addr=None,
@@ -310,13 +346,19 @@ class UCXListener(Listener):
     encrypted = UCXConnector.encrypted
 
     def __init__(
-        self, address: str, comm_handler: None, deserialize=False, **connection_args
+        self,
+        address: str,
+        comm_handler: None,
+        deserialize=False,
+        allow_offload=True,
+        **connection_args
     ):
         if not address.startswith("ucx"):
             address = "ucx://" + address
         self.ip, self._input_port = parse_host_port(address, default_port=0)
         self.comm_handler = comm_handler
         self.deserialize = deserialize
+        self.allow_offload = allow_offload
         self._ep = None  # type: ucp.Endpoint
         self.ucp_server = None
         self.connection_args = connection_args
@@ -337,11 +379,17 @@ class UCXListener(Listener):
                 peer_addr=self.address,
                 deserialize=self.deserialize,
             )
+            ucx.allow_offload = self.allow_offload
+            try:
+                await self.on_connection(ucx)
+            except CommClosedError:
+                logger.debug("Connection closed before handshake completed")
+                return
             if self.comm_handler:
                 await self.comm_handler(ucx)
 
         init_once()
-        self.ucp_server = ucp.create_listener(serve_forever, port=self._input_port)
+        self.ucp_server = ucx_create_listener(serve_forever, port=self._input_port)
 
     def stop(self):
         self.ucp_server = None
@@ -407,7 +455,7 @@ def _scrub_ucx_config():
 
     # configuration of UCX can happen in two ways:
     # 1) high level on/off flags which correspond to UCX configuration
-    # 2) explicity defined UCX configuration flags
+    # 2) explicitly defined UCX configuration flags
 
     # import does not initialize ucp -- this will occur outside this function
     from ucp import get_config
