@@ -14,7 +14,7 @@ from bokeh.models import (
 from tornado import escape
 from dask.utils import format_bytes
 from distributed.utils import log_errors
-from distributed.dashboard.components.scheduler import BOKEH_THEME, TICKS_1024
+from distributed.dashboard.components.scheduler import BOKEH_THEME, TICKS_1024, env
 from distributed.dashboard.utils import without_property_validation, update
 
 
@@ -22,8 +22,10 @@ try:
     import pynvml
 
     pynvml.nvmlInit()
+
+    NVML_ENABLED = True
 except Exception:
-    pass
+    NVML_ENABLED = False
 
 
 class GPUCurrentLoad(DashboardComponent):
@@ -131,26 +133,23 @@ class GPUCurrentLoad(DashboardComponent):
             memory_total = 0
             memory_max = 0
             worker = []
-            i = 0
 
-            for ws in workers:
-                info = ws.extra["gpu"]
+            for idx, ws in enumerate(workers):
+                try:
+                    info = ws.extra["gpu"]
+                except KeyError:
+                    continue
                 metrics = ws.metrics["gpu"]
-                for j, (u, mem_used, mem_total) in enumerate(
-                    zip(
-                        metrics["utilization"],
-                        metrics["memory-used"],
-                        info["memory-total"],
-                    )
-                ):
-                    memory_max = max(memory_max, mem_total)
-                    memory_total += mem_total
-                    utilization.append(int(u))
-                    memory.append(mem_used)
-                    worker.append(ws.address)
-                    gpu_index.append(j)
-                    y.append(i)
-                    i += 1
+                u = metrics["utilization"]
+                mem_used = metrics["memory-used"]
+                mem_total = info["memory-total"]
+                memory_max = max(memory_max, mem_total)
+                memory_total += mem_total
+                utilization.append(int(u))
+                memory.append(mem_used)
+                worker.append(ws.address)
+                gpu_index.append(idx)
+                y.append(idx)
 
             memory_text = [format_bytes(m) for m in memory]
 
@@ -176,16 +175,32 @@ class GPUCurrentLoad(DashboardComponent):
 
 
 def gpu_memory_doc(scheduler, extra, doc):
-    gpu_load = GPUCurrentLoad(scheduler, sizing_mode="stretch_both")
-    gpu_load.update()
-    add_periodic_callback(doc, gpu_load, 100)
-    doc.add_root(gpu_load.memory_figure)
-    doc.theme = BOKEH_THEME
+    with log_errors():
+        gpu_load = GPUCurrentLoad(scheduler, sizing_mode="stretch_both")
+        gpu_load.update()
+        add_periodic_callback(doc, gpu_load, 100)
+        doc.add_root(gpu_load.memory_figure)
+        doc.theme = BOKEH_THEME
 
 
 def gpu_utilization_doc(scheduler, extra, doc):
-    gpu_load = GPUCurrentLoad(scheduler, sizing_mode="stretch_both")
-    gpu_load.update()
-    add_periodic_callback(doc, gpu_load, 100)
-    doc.add_root(gpu_load.utilization_figure)
-    doc.theme = BOKEH_THEME
+    with log_errors():
+        gpu_load = GPUCurrentLoad(scheduler, sizing_mode="stretch_both")
+        gpu_load.update()
+        add_periodic_callback(doc, gpu_load, 100)
+        doc.add_root(gpu_load.utilization_figure)
+        doc.theme = BOKEH_THEME
+
+
+def gpu_doc(scheduler, extra, doc):
+    with log_errors():
+        gpu_load = GPUCurrentLoad(scheduler, sizing_mode="stretch_both")
+        gpu_load.update()
+        add_periodic_callback(doc, gpu_load, 100)
+        doc.add_root(gpu_load.memory_figure)
+        doc.add_root(gpu_load.utilization_figure)
+
+        doc.title = "Dask: GPU"
+        doc.theme = BOKEH_THEME
+        doc.template = env.get_template("gpu.html")
+        doc.template_variables.update(extra)
