@@ -32,14 +32,26 @@ from .components.scheduler import (
     individual_bandwidth_types_doc,
     individual_bandwidth_workers_doc,
     individual_memory_by_key_doc,
+    individual_compute_time_per_key_doc,
+    individual_aggregate_time_per_action_doc,
+    individual_systemmonitor_doc,
 )
 from .worker import counters_doc
+from .components.nvml import (
+    NVML_ENABLED,
+    gpu_memory_doc,
+    gpu_utilization_doc,
+    gpu_doc,
+)  # noqa: 1708
 
 
 template_variables = {
     "pages": ["status", "workers", "tasks", "system", "profile", "graph", "info"]
 }
 
+if NVML_ENABLED:
+    template_variables["pages"].insert(4, "gpu")
+
 
 def connect(application, http_server, scheduler, prefix=""):
     bokeh_app = BokehApplication(
@@ -71,6 +83,7 @@ applications = {
     "/profile": profile_doc,
     "/profile-server": profile_server_doc,
     "/graph": graph_doc,
+    "/gpu": gpu_doc,
     "/individual-task-stream": individual_task_stream_doc,
     "/individual-progress": individual_progress_doc,
     "/individual-graph": individual_graph_doc,
@@ -83,14 +96,9 @@ applications = {
     "/individual-bandwidth-types": individual_bandwidth_types_doc,
     "/individual-bandwidth-workers": individual_bandwidth_workers_doc,
     "/individual-memory-by-key": individual_memory_by_key_doc,
+    "/individual-compute-time-per-key": individual_compute_time_per_key_doc,
+    "/individual-aggregate-time-per-action": individual_aggregate_time_per_action_doc,
+    "/individual-gpu-memory": gpu_memory_doc,
+    "/individual-gpu-utilization": gpu_utilization_doc,
+    "/individual-scheduler-system": individual_systemmonitor_doc,
 }
-
-try:
-    import pynvml  # noqa: 1708
-except ImportError:
-    pass
-else:
-    from .components.nvml import gpu_memory_doc, gpu_utilization_doc  # noqa: 1708
-
-    applications["/individual-gpu-memory"] = gpu_memory_doc
-    applications["/individual-gpu-utilization"] = gpu_utilization_doc
