@@ -5,6 +5,7 @@ import copy
 import logging
 import math
 import weakref
+import warnings
 
 import dask
 from tornado import gen
@@ -36,25 +37,45 @@ class ProcessInterface:
     It should implement the methods below, like ``start`` and ``close``
     """
 
+    @property
+    def status(self):
+        return self._status
+
+    @status.setter
+    def status(self, new_status):
+        if isinstance(new_status, Status):
+            self._status = new_status
+        elif isinstance(new_status, str) or new_status is None:
+            warnings.warn(
+                f"Since distributed 2.19 `.status` is now an Enum, please assign `Status.{new_status}`",
+                PendingDeprecationWarning,
+                stacklevel=1,
+            )
+            corresponding_enum_variants = [s for s in Status if s.value == new_status]
+            assert len(corresponding_enum_variants) == 1
+            self._status = corresponding_enum_variants[0]
+        else:
+            raise TypeError(f"expected Status or str, got {new_status}")
+
     def __init__(self, scheduler=None, name=None):
         self.address = getattr(self, "address", None)
         self.external_address = None
         self.lock = asyncio.Lock()
-        self.status = "created"
+        self.status = Status.created
         self._event_finished = asyncio.Event()
 
     def __await__(self):
         async def _():
             async with self.lock:
-                if self.status == "created":
+                if self.status == Status.created:
                     await self.start()
-                    assert self.status == "running"
+                    assert self.status == Status.running
             return self
 
         return _().__await__()
 
     async def start(self):
-        """ Submit the process to the resource manager
+        """Submit the process to the resource manager
 
         For workers this doesn't have to wait until the process actually starts,
         but can return once the resource manager has the request, and will work
@@ -63,17 +84,17 @@ class ProcessInterface:
         For the scheduler we will expect the scheduler's ``.address`` attribute
         to be avaialble after this completes.
         """
-        self.status = "running"
+        self.status = Status.running
 
     async def close(self):
-        """ Close the process
+        """Close the process
 
         This will be called by the Cluster object when we scale down a node,
         but only after we ask the Scheduler to close the worker gracefully.
         This method should kill the process a bit more forcefully and does not
         need to worry about shutting down gracefully
         """
-        self.status = "closed"
+        self.status = Status.closed
         self._event_finished.set()
 
     async def finished(self):
@@ -105,7 +126,7 @@ class NoOpAwaitable:
 
 
 class SpecCluster(Cluster):
-    """ Cluster that requires a full specification of workers
+    """Cluster that requires a full specification of workers
 
     The SpecCluster class expects a full specification of the Scheduler and
     Workers to use.  It removes any handling of user inputs (like threads vs
@@ -224,16 +245,17 @@ class SpecCluster(Cluster):
         security=None,
         silence_logs=False,
         name=None,
+        shutdown_on_close=True,
     ):
         self._created = weakref.WeakSet()
 
         self.scheduler_spec = copy.copy(scheduler)
         self.worker_spec = copy.copy(workers) or {}
         self.new_spec = copy.copy(worker)
+        self.scheduler = None
         self.workers = {}
         self._i = 0
         self.security = security or Security()
-        self.scheduler_comm = None
         self._futures = set()
 
         if silence_logs:
@@ -248,8 +270,12 @@ class SpecCluster(Cluster):
         self._instances.add(self)
         self._correct_state_waiting = None
         self._name = name or type(self).__name__
+        self.shutdown_on_close = shutdown_on_close
 
-        super().__init__(asynchronous=asynchronous)
+        super().__init__(
+            asynchronous=asynchronous,
+            name=name,
+        )
 
         if not self.asynchronous:
             self._loop_runner.start()
@@ -257,14 +283,15 @@ class SpecCluster(Cluster):
             self.sync(self._correct_state)
 
     async def _start(self):
-        while self.status == "starting":
+        while self.status == Status.starting:
             await asyncio.sleep(0.01)
-        if self.status == "running":
+        if self.status == Status.running:
             return
-        if self.status == "closed":
+        if self.status == Status.closed:
             raise ValueError("Cluster is closed")
 
         self._lock = asyncio.Lock()
+        self.status = Status.starting
 
         if self.scheduler_spec is None:
             try:
@@ -275,18 +302,23 @@ class SpecCluster(Cluster):
                 options = {"dashboard": True}
             self.scheduler_spec = {"cls": Scheduler, "options": options}
 
-        cls = self.scheduler_spec["cls"]
-        if isinstance(cls, str):
-            cls = import_term(cls)
-        self.scheduler = cls(**self.scheduler_spec.get("options", {}))
-
-        self.status = "starting"
-        self.scheduler = await self.scheduler
+        # Check if scheduler has already been created by a subclass
+        if self.scheduler is None:
+            cls = self.scheduler_spec["cls"]
+            if isinstance(cls, str):
+                cls = import_term(cls)
+            self.scheduler = cls(**self.scheduler_spec.get("options", {}))
+            self.scheduler = await self.scheduler
         self.scheduler_comm = rpc(
             getattr(self.scheduler, "external_address", None) or self.scheduler.address,
             connection_args=self.security.get_connection_args("client"),
         )
-        await super()._start()
+        try:
+            await super()._start()
+        except Exception as e:
+            self.status = Status.failed
+            await self._close()
+            raise RuntimeError(f"Cluster failed to start. {str(e)}") from e
 
     def _correct_state(self):
         if self._correct_state_waiting:
@@ -360,7 +392,7 @@ class SpecCluster(Cluster):
 
     def __await__(self):
         async def _():
-            if self.status == "created":
+            if self.status == Status.created:
                 await self._start()
             await self.scheduler
             await self._correct_state()
@@ -371,13 +403,12 @@ class SpecCluster(Cluster):
         return _().__await__()
 
     async def _close(self):
-        while self.status == "closing":
+        while self.status == Status.closing:
             await asyncio.sleep(0.1)
-        if self.status == "closed":
+        if self.status == Status.closed:
             return
-        if self.status == "running":
-            self.status = "closing"
-
+        if self.status == Status.running or self.status == Status.failed:
+            self.status = Status.closing
             self.scale(0)
             await self._correct_state()
             for future in self._futures:
@@ -403,7 +434,7 @@ class SpecCluster(Cluster):
     async def __aenter__(self):
         await self
         await self._correct_state()
-        assert self.status == "running"
+        assert self.status == Status.running
         return self
 
     def __exit__(self, typ, value, traceback):
@@ -454,7 +485,7 @@ class SpecCluster(Cluster):
         while len(self.worker_spec) > n:
             self.worker_spec.popitem()
 
-        if self.status not in ("closing", "closed"):
+        if self.status not in (Status.closing, Status.closed):
             while len(self.worker_spec) < n:
                 self.worker_spec.update(self.new_worker_spec())
 
@@ -464,7 +495,7 @@ class SpecCluster(Cluster):
             return NoOpAwaitable()
 
     def _new_worker_name(self, worker_number):
-        """ Returns new worker name.
+        """Returns new worker name.
 
         This can be overriden in SpecCluster derived classes to customise the
         worker names.
@@ -472,7 +503,7 @@ class SpecCluster(Cluster):
         return worker_number
 
     def new_worker_spec(self):
-        """ Return name and spec for the next worker
+        """Return name and spec for the next worker
 
         Returns
         -------
@@ -548,7 +579,7 @@ class SpecCluster(Cluster):
         maximum_memory: str = None,
         **kwargs,
     ) -> Adaptive:
-        """ Turn on adaptivity
+        """Turn on adaptivity
 
         This scales Dask clusters automatically based on scheduler activity.
 
@@ -598,6 +629,11 @@ class SpecCluster(Cluster):
 
         return super().adapt(*args, minimum=minimum, maximum=maximum, **kwargs)
 
+    @classmethod
+    def from_name(cls, name: str):
+        """Create an instance of this class to represent an existing cluster by name."""
+        raise NotImplementedError()
+
 
 async def run_spec(spec: dict, *args):
     workers = {}
@@ -617,6 +653,7 @@ async def run_spec(spec: dict, *args):
 @atexit.register
 def close_clusters():
     for cluster in list(SpecCluster._instances):
-        with suppress(gen.TimeoutError, TimeoutError):
-            if cluster.status != "closed":
-                cluster.close(timeout=10)
+        if cluster.shutdown_on_close:
+            with suppress(gen.TimeoutError, TimeoutError):
+                if cluster.status != Status.closed:
+                    cluster.close(timeout=10)
