@@ -17,7 +17,7 @@ import pytest
 from dask.system import CPU_COUNT
 from distributed import Client, Worker, Nanny, get_client
 from distributed.core import Status
-from distributed.deploy.local import LocalCluster, nprocesses_nthreads
+from distributed.deploy.local import LocalCluster
 from distributed.metrics import time
 from distributed.system import MEMORY_LIMIT
 from distributed.utils_test import (  # noqa: F401
@@ -83,7 +83,7 @@ def test_close_twice():
 
 def test_procs():
     with LocalCluster(
-        2,
+        n_workers=2,
         scheduler_port=0,
         processes=False,
         threads_per_worker=3,
@@ -98,7 +98,7 @@ def test_procs():
         repr(c)
 
     with LocalCluster(
-        2,
+        n_workers=2,
         scheduler_port=0,
         processes=True,
         threads_per_worker=3,
@@ -434,7 +434,7 @@ def test_blocks_until_full(loop):
 @pytest.mark.asyncio
 async def test_scale_up_and_down():
     async with LocalCluster(
-        0,
+        n_workers=0,
         scheduler_port=0,
         processes=False,
         silence_logs=False,
@@ -763,7 +763,7 @@ async def test_scale_retires_workers():
 
     loop = IOLoop.current()
     cluster = await MyCluster(
-        0,
+        n_workers=0,
         scheduler_port=0,
         processes=False,
         silence_logs=False,
@@ -815,19 +815,6 @@ def test_local_tls_restart(loop):
             assert workers_before != workers_after
 
 
-def test_default_process_thread_breakdown():
-    assert nprocesses_nthreads(1) == (1, 1)
-    assert nprocesses_nthreads(4) == (4, 1)
-    assert nprocesses_nthreads(5) == (5, 1)
-    assert nprocesses_nthreads(8) == (4, 2)
-    assert nprocesses_nthreads(12) in ((6, 2), (4, 3))
-    assert nprocesses_nthreads(20) == (5, 4)
-    assert nprocesses_nthreads(24) in ((6, 4), (8, 3))
-    assert nprocesses_nthreads(32) == (8, 4)
-    assert nprocesses_nthreads(40) in ((8, 5), (10, 4))
-    assert nprocesses_nthreads(80) in ((10, 8), (16, 5))
-
-
 def test_asynchronous_property(loop):
     with LocalCluster(
         4,
@@ -986,6 +973,8 @@ async def test_repr(cleanup):
         memory_limit="2GB",
         asynchronous=True,
     ) as cluster:
+        async with Client(cluster, asynchronous=True) as client:
+            await client.wait_for_workers(2)
         text = repr(cluster)
         assert "workers=2" in text
         assert cluster.scheduler_address in text
@@ -1031,9 +1020,6 @@ async def test_capture_security(cleanup, temporary):
 
 
 @pytest.mark.asyncio
-@pytest.mark.skipif(
-    sys.version_info < (3, 7), reason="asyncio.all_tasks not implemented"
-)
 async def test_no_danglng_asyncio_tasks(cleanup):
     start = asyncio.all_tasks()
     async with LocalCluster(asynchronous=True, processes=False):
@@ -1050,3 +1036,28 @@ async def test_async_with():
         assert w
 
     assert not w
+
+
+@pytest.mark.asyncio
+async def test_no_workers(cleanup):
+    async with Client(
+        n_workers=0, silence_logs=False, dashboard_address=None, asynchronous=True
+    ) as c:
+        pass
+
+
+@pytest.mark.asyncio
+async def test_cluster_names():
+    async with LocalCluster(processes=False, asynchronous=True) as unnamed_cluster:
+        async with LocalCluster(
+            processes=False, asynchronous=True, name="mycluster"
+        ) as named_cluster:
+            assert isinstance(unnamed_cluster.name, str)
+            assert isinstance(named_cluster.name, str)
+            assert named_cluster.name == "mycluster"
+            assert unnamed_cluster == unnamed_cluster
+            assert named_cluster == named_cluster
+            assert unnamed_cluster != named_cluster
+
+        async with LocalCluster(processes=False, asynchronous=True) as unnamed_cluster2:
+            assert unnamed_cluster2 != unnamed_cluster
