@@ -1,4 +1,3 @@
-import dask
 import pytest
 
 from distributed.protocol import loads, dumps, msgpack, maybe_compress, to_serialize
@@ -27,8 +26,9 @@ def test_compression_2():
     pytest.importorskip("lz4")
     np = pytest.importorskip("numpy")
     x = np.random.random(10000)
-    header, payload = dumps(x.tobytes())
-    assert not header or not msgpack.loads(header, encoding="utf8").get("compression")
+    msg = dumps(to_serialize(x.tobytes()))
+    compression = msgpack.loads(msg[1]).get("compression")
+    assert all(c is None for c in compression)
 
 
 def test_compression_without_deserialization():
@@ -66,16 +66,15 @@ def test_maybe_compress(lib, compression):
 
     try_converters = [bytes, memoryview]
 
-    with dask.config.set({"distributed.comm.compression": compression}):
-        for f in try_converters:
-            payload = b"123"
-            assert maybe_compress(f(payload)) == (None, payload)
+    for f in try_converters:
+        payload = b"123"
+        assert maybe_compress(f(payload), compression=compression) == (None, payload)
 
-            payload = b"0" * 10000
-            rc, rd = maybe_compress(f(payload))
-            # For some reason compressing memoryviews can force blosc...
-            assert rc in (compression, "blosc")
-            assert compressions[rc]["decompress"](rd) == payload
+        payload = b"0" * 10000
+        rc, rd = maybe_compress(f(payload), compression=compression)
+        # For some reason compressing memoryviews can force blosc...
+        assert rc in (compression, "blosc")
+        assert compressions[rc]["decompress"](rd) == payload
 
 
 def test_maybe_compress_sample():
@@ -89,14 +88,13 @@ def test_maybe_compress_sample():
 
 def test_large_bytes():
     for tp in (bytes, bytearray):
-        msg = {"x": tp(b"0" * 1000000), "y": 1}
+        msg = {"x": to_serialize(tp(b"0" * 1000000)), "y": 1}
         frames = dumps(msg)
+        msg["x"] = msg["x"].data
         assert loads(frames) == msg
         assert len(frames[0]) < 1000
         assert len(frames[1]) < 1000
 
-        assert loads(frames, deserialize=False) == msg
-
 
 @pytest.mark.slow
 def test_large_messages():
