@@ -1,6 +1,6 @@
 from collections import defaultdict, deque
 import logging
-from math import log
+from math import log2
 from time import time
 
 from tornado.ioloop import PeriodicCallback
@@ -14,7 +14,6 @@ from .utils import log_errors, parse_timedelta
 from tlz import topk
 
 LATENCY = 10e-3
-log_2 = log(2)
 
 logger = logging.getLogger(__name__)
 
@@ -57,9 +56,8 @@ class WorkStealing(SchedulerPlugin):
 
         self.scheduler.stream_handlers["steal-response"] = self.move_task_confirm
 
-    @property
-    def log(self):
-        return self.scheduler.events["stealing"]
+    def log(self, msg):
+        return self.scheduler.log_event("stealing", msg)
 
     def add_worker(self, scheduler=None, worker=None):
         self.stealable[worker] = [set() for i in range(15)]
@@ -73,21 +71,20 @@ class WorkStealing(SchedulerPlugin):
     def transition(
         self, key, start, finish, compute_start=None, compute_stop=None, *args, **kwargs
     ):
-        ts = self.scheduler.tasks[key]
         if finish == "processing":
+            ts = self.scheduler.tasks[key]
             self.put_key_in_stealable(ts)
-
-        if start == "processing":
+        elif start == "processing":
+            ts = self.scheduler.tasks[key]
             self.remove_key_from_stealable(ts)
             if finish != "memory":
                 self.in_flight.pop(ts, None)
 
     def put_key_in_stealable(self, ts):
-        ws = ts.processing_on
-        worker = ws.address
         cost_multiplier, level = self.steal_time_ratio(ts)
-        self.log.append(("add-stealable", ts.key, worker, level))
         if cost_multiplier is not None:
+            ws = ts.processing_on
+            worker = ws.address
             self.stealable_all[level].add(ts)
             self.stealable[worker][level].add(ts)
             self.key_stealable[ts] = (worker, level)
@@ -98,7 +95,6 @@ class WorkStealing(SchedulerPlugin):
             return
 
         worker, level = result
-        self.log.append(("remove-stealable", ts.key, worker, level))
         try:
             self.stealable[worker][level].remove(ts)
         except KeyError:
@@ -109,11 +105,10 @@ class WorkStealing(SchedulerPlugin):
             pass
 
     def steal_time_ratio(self, ts):
-        """ The compute to communication time ratio of a key
+        """The compute to communication time ratio of a key
 
         Returns
         -------
-
         cost_multiplier: The increased cost from moving this task as a factor.
         For example a result of zero implies a task without dependencies.
         level: The location within a stealable list to place this value
@@ -121,28 +116,31 @@ class WorkStealing(SchedulerPlugin):
         if not ts.dependencies:  # no dependencies fast path
             return 0, 0
 
-        nbytes = sum(dep.get_nbytes() for dep in ts.dependencies)
-
-        transfer_time = nbytes / self.scheduler.bandwidth + LATENCY
         split = ts.prefix.name
         if split in fast_tasks:
             return None, None
+
         ws = ts.processing_on
         compute_time = ws.processing[ts]
         if compute_time < 0.005:  # 5ms, just give up
             return None, None
+
+        nbytes = ts.get_nbytes_deps()
+        transfer_time = nbytes / self.scheduler.bandwidth + LATENCY
         cost_multiplier = transfer_time / compute_time
         if cost_multiplier > 100:
             return None, None
 
-        level = int(round(log(cost_multiplier) / log_2 + 6, 0))
-        level = max(1, level)
+        level = int(round(log2(cost_multiplier) + 6))
+        if level < 1:
+            level = 1
+
         return cost_multiplier, level
 
     def move_task_request(self, ts, victim, thief):
         try:
             if self.scheduler.validate:
-                if victim is not ts.processing_on:
+                if victim is not ts.processing_on and LOG_PDB:
                     import pdb
 
                     pdb.set_trace()
@@ -187,7 +185,7 @@ class WorkStealing(SchedulerPlugin):
                 pdb.set_trace()
             raise
 
-    def move_task_confirm(self, key=None, worker=None, state=None):
+    async def move_task_confirm(self, key=None, worker=None, state=None):
         try:
             try:
                 ts = self.scheduler.tasks[key]
@@ -232,9 +230,7 @@ class WorkStealing(SchedulerPlugin):
 
             # Victim had already started execution, reverse stealing
             if state in ("memory", "executing", "long-running", None):
-                self.log.append(
-                    ("already-computing", key, victim.address, thief.address)
-                )
+                self.log(("already-computing", key, victim.address, thief.address))
                 self.scheduler.check_idle_saturated(thief)
                 self.scheduler.check_idle_saturated(victim)
 
@@ -254,10 +250,10 @@ class WorkStealing(SchedulerPlugin):
                 self.put_key_in_stealable(ts)
 
                 try:
-                    self.scheduler.send_task_to_worker(thief.address, key)
+                    self.scheduler.send_task_to_worker(thief.address, ts)
                 except CommClosedError:
-                    self.scheduler.remove_worker(thief.address)
-                self.log.append(("confirm", key, victim.address, thief.address))
+                    await self.scheduler.remove_worker(thief.address)
+                self.log(("confirm", key, victim.address, thief.address))
             else:
                 raise ValueError("Unexpected task state: %s" % state)
         except Exception as e:
@@ -306,7 +302,7 @@ class WorkStealing(SchedulerPlugin):
 
         with log_errors():
             i = 0
-            idle = s.idle
+            idle = s.idle.values()
             saturated = s.saturated
             if not idle or len(idle) == len(s.workers):
                 return
@@ -392,7 +388,7 @@ class WorkStealing(SchedulerPlugin):
                         )
 
             if log:
-                self.log.append(log)
+                self.log(log)
                 self.count += 1
             stop = time()
             if s.digests:
@@ -410,7 +406,7 @@ class WorkStealing(SchedulerPlugin):
     def story(self, *keys):
         keys = set(keys)
         out = []
-        for L in self.log:
+        for _, L in self.scheduler.get_event("stealing"):
             if not isinstance(L, list):
                 L = [L]
             for t in L:
