@@ -1,4 +1,6 @@
+import asyncio
 import itertools
+import logging
 import random
 import sys
 import weakref
@@ -23,7 +25,6 @@ from distributed.utils_test import (
     slowinc,
 )
 from tlz import concat, sliding_window
-from tornado import gen
 
 # Most tests here are timing-dependent
 setup_module = nodebug_setup_module
@@ -33,71 +34,71 @@ teardown_module = nodebug_teardown_module
 @pytest.mark.skipif(
     not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
 )
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 2), ("127.0.0.2", 2)], timeout=20)
-def test_work_stealing(c, s, a, b):
-    [x] = yield c._scatter([1], workers=a.address)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 2), ("127.0.0.2", 2)])
+async def test_work_stealing(c, s, a, b):
+    [x] = await c._scatter([1], workers=a.address)
     futures = c.map(slowadd, range(50), [x] * 50)
-    yield wait(futures)
+    await wait(futures)
     assert len(a.data) > 10
     assert len(b.data) > 10
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_dont_steal_expensive_data_fast_computation(c, s, a, b):
+async def test_dont_steal_expensive_data_fast_computation(c, s, a, b):
     np = pytest.importorskip("numpy")
     x = c.submit(np.arange, 1000000, workers=a.address)
-    yield wait([x])
+    await wait([x])
     future = c.submit(np.sum, [1], workers=a.address)  # learn that sum is fast
-    yield wait([future])
+    await wait([future])
 
     cheap = [
         c.submit(np.sum, x, pure=False, workers=a.address, allow_other_workers=True)
         for i in range(10)
     ]
-    yield wait(cheap)
+    await wait(cheap)
     assert len(s.who_has[x.key]) == 1
     assert len(b.data) == 0
     assert len(a.data) == 12
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_steal_cheap_data_slow_computation(c, s, a, b):
+async def test_steal_cheap_data_slow_computation(c, s, a, b):
     x = c.submit(slowinc, 100, delay=0.1)  # learn that slowinc is slow
-    yield wait(x)
+    await wait(x)
 
     futures = c.map(
         slowinc, range(10), delay=0.1, workers=a.address, allow_other_workers=True
     )
-    yield wait(futures)
+    await wait(futures)
     assert abs(len(a.data) - len(b.data)) <= 5
 
 
-@pytest.mark.avoid_travis
+@pytest.mark.avoid_ci
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_steal_expensive_data_slow_computation(c, s, a, b):
+async def test_steal_expensive_data_slow_computation(c, s, a, b):
     np = pytest.importorskip("numpy")
 
     x = c.submit(slowinc, 100, delay=0.2, workers=a.address)
-    yield wait(x)  # learn that slowinc is slow
+    await wait(x)  # learn that slowinc is slow
 
     x = c.submit(np.arange, 1000000, workers=a.address)  # put expensive data
-    yield wait(x)
+    await wait(x)
 
     slow = [c.submit(slowinc, x, delay=0.1, pure=False) for i in range(20)]
-    yield wait(slow)
+    await wait(slow)
     assert len(s.who_has[x.key]) > 1
 
     assert b.data  # not empty
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 10)
-def test_worksteal_many_thieves(c, s, *workers):
+async def test_worksteal_many_thieves(c, s, *workers):
     x = c.submit(slowinc, -1, delay=0.1)
-    yield x
+    await x
 
     xs = c.map(slowinc, [x] * 100, pure=False, delay=0.1)
 
-    yield wait(xs)
+    await wait(xs)
 
     for w, keys in s.has_what.items():
         assert 2 < len(keys) < 30
@@ -106,32 +107,32 @@ def test_worksteal_many_thieves(c, s, *workers):
     assert sum(map(len, s.has_what.values())) < 150
 
 
+@pytest.mark.flaky(reruns=10, reruns_delay=5, reason="GH#3574")
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_dont_steal_unknown_functions(c, s, a, b):
-    futures = c.map(inc, [1, 2], workers=a.address, allow_other_workers=True)
-    yield wait(futures)
-    assert len(a.data) == 2, [len(a.data), len(b.data)]
-    assert len(b.data) == 0, [len(a.data), len(b.data)]
+async def test_dont_steal_unknown_functions(c, s, a, b):
+    futures = c.map(inc, range(100), workers=a.address, allow_other_workers=True)
+    await wait(futures)
+    assert len(a.data) >= 95, [len(a.data), len(b.data)]
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_eventually_steal_unknown_functions(c, s, a, b):
+async def test_eventually_steal_unknown_functions(c, s, a, b):
     futures = c.map(
         slowinc, range(10), delay=0.1, workers=a.address, allow_other_workers=True
     )
-    yield wait(futures)
-    assert len(a.data) >= 3
-    assert len(b.data) >= 3
+    await wait(futures)
+    assert len(a.data) >= 3, [len(a.data), len(b.data)]
+    assert len(b.data) >= 3, [len(a.data), len(b.data)]
 
 
 @pytest.mark.skip(reason="")
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 3)
-def test_steal_related_tasks(e, s, a, b, c):
+async def test_steal_related_tasks(e, s, a, b, c):
     futures = e.map(
         slowinc, range(20), delay=0.05, workers=a.address, allow_other_workers=True
     )
 
-    yield wait(futures)
+    await wait(futures)
 
     nearby = 0
     for f1, f2 in sliding_window(2, futures):
@@ -149,7 +150,7 @@ async def test_dont_steal_fast_tasks_compute_time(c, s, *workers):
     def do_nothing(x, y=None):
         pass
 
-    # execute and meassure runtime once
+    # execute and measure runtime once
     await wait(c.submit(do_nothing, 1))
 
     futures = c.map(do_nothing, range(1000), y=x)
@@ -166,7 +167,7 @@ async def test_dont_steal_fast_tasks_blacklist(c, s, a, b):
     x = c.submit(slowinc, 1, workers=[b.address])
 
     # If the blacklist of fast tasks is tracked somewhere else, this needs to be
-    # changed. This test requies *any* key which is blacklisted.
+    # changed. This test requires *any* key which is blacklisted.
     from distributed.stealing import fast_tasks
 
     blacklisted_key = next(iter(fast_tasks))
@@ -174,7 +175,7 @@ async def test_dont_steal_fast_tasks_blacklist(c, s, a, b):
     def fast_blacklisted(x, y=None):
         # The task should observe a certain computation time such that we can
         # ensure that it is not stolen due to the blacklisting. If it is too
-        # fast, the standard mechansim shouldn't allow stealing
+        # fast, the standard mechanism shouldn't allow stealing
         import time
 
         time.sleep(0.01)
@@ -197,18 +198,18 @@ async def test_dont_steal_fast_tasks_blacklist(c, s, a, b):
     assert len(s.has_what[b.address]) == 1
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)], timeout=20)
-def test_new_worker_steals(c, s, a):
-    yield wait(c.submit(slowinc, 1, delay=0.01))
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)])
+async def test_new_worker_steals(c, s, a):
+    await wait(c.submit(slowinc, 1, delay=0.01))
 
     futures = c.map(slowinc, range(100), delay=0.05)
     total = c.submit(sum, futures)
-    while len(a.task_state) < 10:
-        yield gen.sleep(0.01)
+    while len(a.tasks) < 10:
+        await asyncio.sleep(0.01)
 
-    b = yield Worker(s.address, loop=s.loop, nthreads=1, memory_limit=MEMORY_LIMIT)
+    b = await Worker(s.address, loop=s.loop, nthreads=1, memory_limit=MEMORY_LIMIT)
 
-    result = yield total
+    result = await total
     assert result == sum(map(inc, range(100)))
 
     for w in [a, b]:
@@ -216,171 +217,171 @@ def test_new_worker_steals(c, s, a):
 
     assert b.data
 
-    yield b.close()
+    await b.close()
 
 
-@gen_cluster(client=True, timeout=20)
-def test_work_steal_no_kwargs(c, s, a, b):
-    yield wait(c.submit(slowinc, 1, delay=0.05))
+@gen_cluster(client=True)
+async def test_work_steal_no_kwargs(c, s, a, b):
+    await wait(c.submit(slowinc, 1, delay=0.05))
 
     futures = c.map(
         slowinc, range(100), workers=a.address, allow_other_workers=True, delay=0.05
     )
 
-    yield wait(futures)
+    await wait(futures)
 
     assert 20 < len(a.data) < 80
     assert 20 < len(b.data) < 80
 
     total = c.submit(sum, futures)
-    result = yield total
+    result = await total
 
     assert result == sum(map(inc, range(100)))
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1), ("127.0.0.1", 2)])
-def test_dont_steal_worker_restrictions(c, s, a, b):
+async def test_dont_steal_worker_restrictions(c, s, a, b):
     future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
-    yield future
+    await future
 
     futures = c.map(slowinc, range(100), delay=0.1, workers=a.address)
 
-    while len(a.task_state) + len(b.task_state) < 100:
-        yield gen.sleep(0.01)
+    while len(a.tasks) + len(b.tasks) < 100:
+        await asyncio.sleep(0.01)
 
-    assert len(a.task_state) == 100
-    assert len(b.task_state) == 0
+    assert len(a.tasks) == 100
+    assert len(b.tasks) == 0
 
     result = s.extensions["stealing"].balance()
 
-    yield gen.sleep(0.1)
+    await asyncio.sleep(0.1)
 
-    assert len(a.task_state) == 100
-    assert len(b.task_state) == 0
+    assert len(a.tasks) == 100
+    assert len(b.tasks) == 0
 
 
 @gen_cluster(
     client=True, nthreads=[("127.0.0.1", 1), ("127.0.0.1", 2), ("127.0.0.1", 2)]
 )
-def test_steal_worker_restrictions(c, s, wa, wb, wc):
+async def test_steal_worker_restrictions(c, s, wa, wb, wc):
     future = c.submit(slowinc, 1, delay=0.1, workers={wa.address, wb.address})
-    yield future
+    await future
 
     ntasks = 100
     futures = c.map(slowinc, range(ntasks), delay=0.1, workers={wa.address, wb.address})
 
-    while sum(len(w.task_state) for w in [wa, wb, wc]) < ntasks:
-        yield gen.sleep(0.01)
+    while sum(len(w.tasks) for w in [wa, wb, wc]) < ntasks:
+        await asyncio.sleep(0.01)
 
-    assert 0 < len(wa.task_state) < ntasks
-    assert 0 < len(wb.task_state) < ntasks
-    assert len(wc.task_state) == 0
+    assert 0 < len(wa.tasks) < ntasks
+    assert 0 < len(wb.tasks) < ntasks
+    assert len(wc.tasks) == 0
 
     s.extensions["stealing"].balance()
 
-    yield gen.sleep(0.1)
+    await asyncio.sleep(0.1)
 
-    assert 0 < len(wa.task_state) < ntasks
-    assert 0 < len(wb.task_state) < ntasks
-    assert len(wc.task_state) == 0
+    assert 0 < len(wa.tasks) < ntasks
+    assert 0 < len(wb.tasks) < ntasks
+    assert len(wc.tasks) == 0
 
 
 @pytest.mark.skipif(
     not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
 )
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1), ("127.0.0.2", 1)])
-def test_dont_steal_host_restrictions(c, s, a, b):
+async def test_dont_steal_host_restrictions(c, s, a, b):
     future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
-    yield future
+    await future
 
     futures = c.map(slowinc, range(100), delay=0.1, workers="127.0.0.1")
-    while len(a.task_state) + len(b.task_state) < 100:
-        yield gen.sleep(0.01)
-    assert len(a.task_state) == 100
-    assert len(b.task_state) == 0
+    while len(a.tasks) + len(b.tasks) < 100:
+        await asyncio.sleep(0.01)
+    assert len(a.tasks) == 100
+    assert len(b.tasks) == 0
 
     result = s.extensions["stealing"].balance()
 
-    yield gen.sleep(0.1)
-    assert len(a.task_state) == 100
-    assert len(b.task_state) == 0
+    await asyncio.sleep(0.1)
+    assert len(a.tasks) == 100
+    assert len(b.tasks) == 0
 
 
 @pytest.mark.skipif(
     not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
 )
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1), ("127.0.0.2", 2)])
-def test_steal_host_restrictions(c, s, wa, wb):
+async def test_steal_host_restrictions(c, s, wa, wb):
     future = c.submit(slowinc, 1, delay=0.10, workers=wa.address)
-    yield future
+    await future
 
     ntasks = 100
     futures = c.map(slowinc, range(ntasks), delay=0.1, workers="127.0.0.1")
-    while len(wa.task_state) < ntasks:
-        yield gen.sleep(0.01)
-    assert len(wa.task_state) == ntasks
-    assert len(wb.task_state) == 0
+    while len(wa.tasks) < ntasks:
+        await asyncio.sleep(0.01)
+    assert len(wa.tasks) == ntasks
+    assert len(wb.tasks) == 0
 
-    wc = yield Worker(s.address, ncores=1)
+    wc = await Worker(s.address, nthreads=1)
 
     start = time()
-    while not wc.task_state or len(wa.task_state) == ntasks:
-        yield gen.sleep(0.01)
+    while not wc.tasks or len(wa.tasks) == ntasks:
+        await asyncio.sleep(0.01)
         assert time() < start + 3
 
-    yield gen.sleep(0.1)
-    assert 0 < len(wa.task_state) < ntasks
-    assert len(wb.task_state) == 0
-    assert 0 < len(wc.task_state) < ntasks
+    await asyncio.sleep(0.1)
+    assert 0 < len(wa.tasks) < ntasks
+    assert len(wb.tasks) == 0
+    assert 0 < len(wc.tasks) < ntasks
 
 
 @gen_cluster(
     client=True, nthreads=[("127.0.0.1", 1, {"resources": {"A": 2}}), ("127.0.0.1", 1)]
 )
-def test_dont_steal_resource_restrictions(c, s, a, b):
+async def test_dont_steal_resource_restrictions(c, s, a, b):
     future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
-    yield future
+    await future
 
     futures = c.map(slowinc, range(100), delay=0.1, resources={"A": 1})
-    while len(a.task_state) + len(b.task_state) < 100:
-        yield gen.sleep(0.01)
-    assert len(a.task_state) == 100
-    assert len(b.task_state) == 0
+    while len(a.tasks) + len(b.tasks) < 100:
+        await asyncio.sleep(0.01)
+    assert len(a.tasks) == 100
+    assert len(b.tasks) == 0
 
     result = s.extensions["stealing"].balance()
 
-    yield gen.sleep(0.1)
-    assert len(a.task_state) == 100
-    assert len(b.task_state) == 0
+    await asyncio.sleep(0.1)
+    assert len(a.tasks) == 100
+    assert len(b.tasks) == 0
 
 
 @gen_cluster(
     client=True, nthreads=[("127.0.0.1", 1, {"resources": {"A": 2}})], timeout=3
 )
-def test_steal_resource_restrictions(c, s, a):
+async def test_steal_resource_restrictions(c, s, a):
     future = c.submit(slowinc, 1, delay=0.10, workers=a.address)
-    yield future
+    await future
 
     futures = c.map(slowinc, range(100), delay=0.2, resources={"A": 1})
-    while len(a.task_state) < 101:
-        yield gen.sleep(0.01)
-    assert len(a.task_state) == 101
+    while len(a.tasks) < 101:
+        await asyncio.sleep(0.01)
+    assert len(a.tasks) == 101
 
-    b = yield Worker(s.address, loop=s.loop, nthreads=1, resources={"A": 4})
+    b = await Worker(s.address, loop=s.loop, nthreads=1, resources={"A": 4})
 
     start = time()
-    while not b.task_state or len(a.task_state) == 101:
-        yield gen.sleep(0.01)
+    while not b.tasks or len(a.tasks) == 101:
+        await asyncio.sleep(0.01)
         assert time() < start + 3
 
-    assert len(b.task_state) > 0
-    assert len(a.task_state) < 101
+    assert len(b.tasks) > 0
+    assert len(a.tasks) < 101
 
-    yield b.close()
+    await b.close()
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 5, timeout=20)
-def test_balance_without_dependencies(c, s, *workers):
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 5)
+async def test_balance_without_dependencies(c, s, *workers):
     s.extensions["stealing"]._pc.callback_time = 20
 
     def slow(x):
@@ -389,19 +390,19 @@ def test_balance_without_dependencies(c, s, *workers):
         return y
 
     futures = c.map(slow, range(100))
-    yield wait(futures)
+    await wait(futures)
 
     durations = [sum(w.data.values()) for w in workers]
     assert max(durations) / min(durations) < 3
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 4)] * 2)
-def test_dont_steal_executing_tasks(c, s, a, b):
+async def test_dont_steal_executing_tasks(c, s, a, b):
     futures = c.map(
         slowinc, range(4), delay=0.1, workers=a.address, allow_other_workers=True
     )
 
-    yield wait(futures)
+    await wait(futures)
     assert len(a.data) == 4
     assert len(b.data) == 0
 
@@ -411,17 +412,17 @@ def test_dont_steal_executing_tasks(c, s, a, b):
     nthreads=[("127.0.0.1", 1)] * 10,
     config={"distributed.scheduler.default-task-durations": {"slowidentity": 0.2}},
 )
-def test_dont_steal_few_saturated_tasks_many_workers(c, s, a, *rest):
+async def test_dont_steal_few_saturated_tasks_many_workers(c, s, a, *rest):
     s.extensions["stealing"]._pc.callback_time = 20
     x = c.submit(mul, b"0", 100000000, workers=a.address)  # 100 MB
-    yield wait(x)
+    await wait(x)
 
     futures = [c.submit(slowidentity, x, pure=False, delay=0.2) for i in range(2)]
 
-    yield wait(futures)
+    await wait(futures)
 
     assert len(a.data) == 3
-    assert not any(w.task_state for w in rest)
+    assert not any(w.tasks for w in rest)
 
 
 @gen_cluster(
@@ -430,16 +431,16 @@ def test_dont_steal_few_saturated_tasks_many_workers(c, s, a, *rest):
     worker_kwargs={"memory_limit": MEMORY_LIMIT},
     config={"distributed.scheduler.default-task-durations": {"slowidentity": 0.2}},
 )
-def test_steal_when_more_tasks(c, s, a, *rest):
+async def test_steal_when_more_tasks(c, s, a, *rest):
     s.extensions["stealing"]._pc.callback_time = 20
     x = c.submit(mul, b"0", 50000000, workers=a.address)  # 50 MB
-    yield wait(x)
+    await wait(x)
 
     futures = [c.submit(slowidentity, x, pure=False, delay=0.2) for i in range(20)]
 
     start = time()
-    while not any(w.task_state for w in rest):
-        yield gen.sleep(0.01)
+    while not any(w.tasks for w in rest):
+        await asyncio.sleep(0.01)
         assert time() < start + 1
 
 
@@ -453,30 +454,30 @@ def test_steal_when_more_tasks(c, s, a, *rest):
         }
     },
 )
-def test_steal_more_attractive_tasks(c, s, a, *rest):
+async def test_steal_more_attractive_tasks(c, s, a, *rest):
     def slow2(x):
         sleep(1)
         return x
 
     s.extensions["stealing"]._pc.callback_time = 20
     x = c.submit(mul, b"0", 100000000, workers=a.address)  # 100 MB
-    yield wait(x)
+    await wait(x)
 
     futures = [c.submit(slowidentity, x, pure=False, delay=0.2) for i in range(10)]
     future = c.submit(slow2, x, priority=-1)
 
-    while not any(w.task_state for w in rest):
-        yield gen.sleep(0.01)
+    while not any(w.tasks for w in rest):
+        await asyncio.sleep(0.01)
 
     # good future moves first
-    assert any(future.key in w.task_state for w in rest)
+    assert any(future.key in w.tasks for w in rest)
 
 
 def func(x):
     sleep(1)
 
 
-def assert_balanced(inp, expected, c, s, *workers):
+async def assert_balanced(inp, expected, c, s, *workers):
     steal = s.extensions["stealing"]
     steal._pc.stop()
 
@@ -488,11 +489,11 @@ def assert_balanced(inp, expected, c, s, *workers):
     for w, ts in zip(workers, inp):
         for t in sorted(ts, reverse=True):
             if t:
-                [dat] = yield c.scatter([next(data_seq)], workers=w.address)
+                [dat] = await c.scatter([next(data_seq)], workers=w.address)
                 ts = s.tasks[dat.key]
                 # Ensure scheduler state stays consistent
                 old_nbytes = ts.nbytes
-                ts.nbytes = s.bandwidth * t
+                ts.nbytes = int(s.bandwidth * t)
                 for ws in ts.who_has:
                     ws.nbytes += ts.nbytes - old_nbytes
             else:
@@ -510,13 +511,13 @@ def assert_balanced(inp, expected, c, s, *workers):
             futures.append(f)
 
     while len(s.rprocessing) < len(futures):
-        yield gen.sleep(0.001)
+        await asyncio.sleep(0.001)
 
     for i in range(10):
         steal.balance()
 
         while steal.in_flight:
-            yield gen.sleep(0.001)
+            await asyncio.sleep(0.001)
 
         result = [
             sorted([int(key_split(k)) for k in s.processing[w.address]], reverse=True)
@@ -562,6 +563,7 @@ def assert_balanced(inp, expected, c, s, *workers):
         pytest.param(
             [[1, 1, 1, 1, 1, 1, 1], [1, 1], [1, 1], [1, 1], []],
             [[1, 1, 1, 1, 1], [1, 1], [1, 1], [1, 1], [1, 1]],
+            # Can't mark as flaky as when it fails it does so every time for some reason
             marks=pytest.mark.xfail(
                 reason="Some uncertainty based on executing stolen task"
             ),
@@ -569,7 +571,9 @@ def assert_balanced(inp, expected, c, s, *workers):
     ],
 )
 def test_balance(inp, expected):
-    test = lambda *args, **kwargs: assert_balanced(inp, expected, *args, **kwargs)
+    async def test(*args, **kwargs):
+        await assert_balanced(inp, expected, *args, **kwargs)
+
     test = gen_cluster(
         client=True,
         nthreads=[("127.0.0.1", 1)] * len(inp),
@@ -582,19 +586,19 @@ def test_balance(inp, expected):
     test()
 
 
-@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2, Worker=Nanny, timeout=20)
-def test_restart(c, s, a, b):
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2, Worker=Nanny)
+async def test_restart(c, s, a, b):
     futures = c.map(
         slowinc, range(100), delay=0.1, workers=a.address, allow_other_workers=True
     )
     while not s.processing[b.worker_address]:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     steal = s.extensions["stealing"]
     assert any(st for st in steal.stealable_all)
     assert any(x for L in steal.stealable.values() for x in L)
 
-    yield c.restart(timeout=10)
+    await c.restart(timeout=10)
 
     assert not any(x for x in steal.stealable_all)
     assert not any(x for L in steal.stealable.values() for x in L)
@@ -604,7 +608,7 @@ def test_restart(c, s, a, b):
     client=True,
     config={"distributed.scheduler.default-task-durations": {"slowadd": 0.001}},
 )
-def test_steal_communication_heavy_tasks(c, s, a, b):
+async def test_steal_communication_heavy_tasks(c, s, a, b):
     steal = s.extensions["stealing"]
     x = c.submit(mul, b"0", int(s.bandwidth), workers=a.address)
     y = c.submit(mul, b"1", int(s.bandwidth), workers=b.address)
@@ -623,29 +627,29 @@ def test_steal_communication_heavy_tasks(c, s, a, b):
     ]
 
     while not any(f.key in s.rprocessing for f in futures):
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     steal.balance()
     while steal.in_flight:
-        yield gen.sleep(0.001)
+        await asyncio.sleep(0.001)
 
     assert s.processing[b.address]
 
 
 @gen_cluster(client=True)
-def test_steal_twice(c, s, a, b):
+async def test_steal_twice(c, s, a, b):
     x = c.submit(inc, 1, workers=a.address)
-    yield wait(x)
+    await wait(x)
 
     futures = [c.submit(slowadd, x, i, delay=0.2) for i in range(100)]
 
     while len(s.tasks) < 100:  # tasks are all allocated
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     # Army of new workers arrives to help
-    workers = yield [Worker(s.address, loop=s.loop) for _ in range(20)]
+    workers = await asyncio.gather(*[Worker(s.address, loop=s.loop) for _ in range(20)])
 
-    yield wait(futures)
+    await wait(futures)
 
     has_what = dict(s.has_what)  # take snapshot
     empty_workers = [w for w, keys in has_what.items() if not len(keys)]
@@ -656,52 +660,79 @@ def test_steal_twice(c, s, a, b):
         )
     assert max(map(len, has_what.values())) < 30
 
-    yield c._close()
-    yield [w.close() for w in workers]
+    assert a.in_flight_tasks == 0
+    assert b.in_flight_tasks == 0
+
+    await c._close()
+    await asyncio.gather(*[w.close() for w in workers])
 
 
 @gen_cluster(client=True)
-def test_dont_steal_executing_tasks(c, s, a, b):
+async def test_dont_steal_executing_tasks(c, s, a, b):
     steal = s.extensions["stealing"]
 
     future = c.submit(slowinc, 1, delay=0.5, workers=a.address)
-    while not a.executing:
-        yield gen.sleep(0.01)
+    while not a.executing_count:
+        await asyncio.sleep(0.01)
 
     steal.move_task_request(
         s.tasks[future.key], s.workers[a.address], s.workers[b.address]
     )
-    yield gen.sleep(0.1)
-    assert future.key in a.executing
-    assert not b.executing
+    await asyncio.sleep(0.1)
+    assert a.tasks[future.key].state == "executing"
+    assert not b.executing_count
+
+
+@gen_cluster(client=True)
+async def test_dont_steal_already_released(c, s, a, b):
+    future = c.submit(slowinc, 1, delay=0.05, workers=a.address)
+    key = future.key
+    await asyncio.sleep(0.05)
+    assert key in a.tasks
+    del future
+    await asyncio.sleep(0.05)
+    # In case the system is slow (e.g. network) ensure that nothing bad happens
+    # if the key was already released
+    assert key not in a.tasks
+    a.steal_request(key)
+    assert a.batched_stream.buffer == [
+        {"op": "steal-response", "key": key, "state": None}
+    ]
+    with captured_logger(
+        logging.getLogger("distributed.stealing"), level=logging.DEBUG
+    ) as stealing_logs:
+        await asyncio.sleep(0.05)
+
+    logs = stealing_logs.getvalue()
+    assert f"Key released between request and confirm: {key}" in logs
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_dont_steal_long_running_tasks(c, s, a, b):
+async def test_dont_steal_long_running_tasks(c, s, a, b):
     def long(delay):
         with worker_client() as c:
             sleep(delay)
 
-    yield c.submit(long, 0.1)  # learn duration
-    yield c.submit(inc, 1)  # learn duration
+    await c.submit(long, 0.1)  # learn duration
+    await c.submit(inc, 1)  # learn duration
 
     long_tasks = c.map(long, [0.5, 0.6], workers=a.address, allow_other_workers=True)
     while sum(map(len, s.processing.values())) < 2:  # let them start
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
 
     start = time()
     while any(t.key in s.extensions["stealing"].key_stealable for t in long_tasks):
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() < start + 1
 
-    na = len(a.executing)
-    nb = len(b.executing)
+    na = a.executing_count
+    nb = b.executing_count
 
     incs = c.map(inc, range(100), workers=a.address, allow_other_workers=True)
 
-    yield gen.sleep(0.2)
+    await asyncio.sleep(0.2)
 
-    yield wait(long_tasks)
+    await wait(long_tasks)
 
     for t in long_tasks:
         assert (
@@ -710,25 +741,21 @@ def test_dont_steal_long_running_tasks(c, s, a, b):
         ) <= 1
 
 
-@pytest.mark.xfail(
-    sys.version_info[:2] == (3, 8),
-    reason="Sporadic failure on Python 3.8",
-    strict=False,
-)
+@pytest.mark.flaky(reruns=10, reruns_delay=5, condition=sys.version_info[:2] == (3, 8))
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 5)] * 2)
-def test_cleanup_repeated_tasks(c, s, a, b):
+async def test_cleanup_repeated_tasks(c, s, a, b):
     class Foo:
         pass
 
     s.extensions["stealing"]._pc.callback_time = 20
-    yield c.submit(slowidentity, -1, delay=0.1)
+    await c.submit(slowidentity, -1, delay=0.1)
     objects = [c.submit(Foo, pure=False, workers=a.address) for _ in range(50)]
 
     x = c.map(
         slowidentity, objects, workers=a.address, allow_other_workers=True, delay=0.05
     )
     del objects
-    yield wait(x)
+    await wait(x)
     assert a.data and b.data
     assert len(a.data) + len(b.data) > 10
     ws = weakref.WeakSet()
@@ -738,7 +765,7 @@ def test_cleanup_repeated_tasks(c, s, a, b):
 
     start = time()
     while a.data or b.data:
-        yield gen.sleep(0.01)
+        await asyncio.sleep(0.01)
         assert time() < start + 1
 
     assert not s.who_has
@@ -748,7 +775,7 @@ def test_cleanup_repeated_tasks(c, s, a, b):
 
 
 @gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
-def test_lose_task(c, s, a, b):
+async def test_lose_task(c, s, a, b):
     with captured_logger("distributed.stealing") as log:
         s.periodic_callbacks["stealing"].interval = 1
         for i in range(100):
@@ -760,7 +787,7 @@ def test_lose_task(c, s, a, b):
                 workers=a.address,
                 allow_other_workers=True,
             )
-            yield gen.sleep(0.01)
+            await asyncio.sleep(0.01)
             del futures
 
     out = log.getvalue()
@@ -768,7 +795,7 @@ def test_lose_task(c, s, a, b):
 
 
 @gen_cluster(client=True)
-def test_worker_stealing_interval(c, s, a, b):
+async def test_worker_stealing_interval(c, s, a, b):
     from distributed.scheduler import WorkStealing
 
     ws = WorkStealing(s)
@@ -782,3 +809,19 @@ def test_worker_stealing_interval(c, s, a, b):
     with dask.config.set({"distributed.scheduler.work-stealing-interval": 2}):
         ws = WorkStealing(s)
     assert ws._pc.callback_time == 2
+
+
+@gen_cluster(client=True)
+async def test_balance_with_longer_task(c, s, a, b):
+    np = pytest.importorskip("numpy")
+
+    await c.submit(slowinc, 0, delay=0)  # scheduler learns that slowinc is very fast
+    x = await c.scatter(np.arange(10000), workers=[a.address])
+    y = c.submit(
+        slowinc, 1, delay=5, workers=[a.address], priority=1
+    )  # a surprisingly long task
+    z = c.submit(
+        inc, x, workers=[a.address], allow_other_workers=True, priority=0
+    )  # a task after y, suggesting a, but open to b
+    await z
+    assert z.key in b.data
