@@ -10,7 +10,6 @@ import sys
 from time import sleep
 import traceback
 
-import numpy as np
 import pytest
 from tornado.ioloop import IOLoop
 
@@ -24,7 +23,6 @@ from distributed.utils import (
     is_kernel,
     is_valid_xml,
     ensure_ip,
-    str_graph,
     truncate_exception,
     get_traceback,
     _maybe_complex,
@@ -40,6 +38,7 @@ from distributed.utils import (
     LoopRunner,
     parse_bytes,
     parse_timedelta,
+    parse_ports,
     warn_on_duration,
     format_dashboard_link,
     LRU,
@@ -109,6 +108,9 @@ def test_sync_timeout(loop_in_thread):
     with pytest.raises(TimeoutError):
         sync(loop_in_thread, asyncio.sleep, 0.5, callback_timeout=0.05)
 
+    with pytest.raises(TimeoutError):
+        sync(loop_in_thread, asyncio.sleep, 0.5, callback_timeout="50ms")
+
 
 def test_sync_closed_loop():
     loop = IOLoop.current()
@@ -191,32 +193,6 @@ def test_get_traceback():
         assert type(tb).__name__ == "traceback"
 
 
-def test_str_graph():
-    dsk = {"x": 1}
-    assert str_graph(dsk) == dsk
-
-    dsk = {("x", 1): (inc, 1)}
-    assert str_graph(dsk) == {str(("x", 1)): (inc, 1)}
-
-    dsk = {("x", 1): (inc, 1), ("x", 2): (inc, ("x", 1))}
-    assert str_graph(dsk) == {
-        str(("x", 1)): (inc, 1),
-        str(("x", 2)): (inc, str(("x", 1))),
-    }
-
-    dsks = [
-        {"x": 1},
-        {("x", 1): (inc, 1), ("x", 2): (inc, ("x", 1))},
-        {("x", 1): (sum, [1, 2, 3]), ("x", 2): (sum, [("x", 1), ("x", 1)])},
-    ]
-    for dsk in dsks:
-        sdsk = str_graph(dsk)
-        keys = list(dsk)
-        skeys = [str(k) for k in keys]
-        assert all(isinstance(k, str) for k in sdsk)
-        assert dask.get(dsk, keys) == dask.get(sdsk, skeys)
-
-
 def test_maybe_complex():
     assert not _maybe_complex(1)
     assert not _maybe_complex("x")
@@ -292,6 +268,7 @@ def test_ensure_bytes():
 
 
 def test_ensure_bytes_ndarray():
+    np = pytest.importorskip("numpy")
     result = ensure_bytes(np.arange(12))
     assert isinstance(result, bytes)
 
@@ -304,6 +281,8 @@ def test_ensure_bytes_pyarrow_buffer():
 
 
 def test_nbytes():
+    np = pytest.importorskip("numpy")
+
     def check(obj, expected):
         assert nbytes(obj) == expected
         assert nbytes(memoryview(obj)) == expected
@@ -597,6 +576,16 @@ def test_format_dashboard_link():
         del os.environ["host"]
 
 
+def test_parse_ports():
+    assert parse_ports(None) == [None]
+    assert parse_ports(23) == [23]
+    assert parse_ports("45") == [45]
+    assert parse_ports("100:103") == [100, 101, 102, 103]
+
+    with pytest.raises(ValueError, match="port_stop must be greater than port_start"):
+        parse_ports("103:100")
+
+
 def test_lru():
 
     l = LRU(maxsize=3)
