@@ -1,6 +1,5 @@
 import asyncio
 from asyncio import TimeoutError
-import atexit
 import click
 from collections import deque, OrderedDict, UserDict
 from concurrent.futures import ThreadPoolExecutor, CancelledError  # noqa: F401
@@ -25,7 +24,6 @@ import threading
 import warnings
 import weakref
 import pkgutil
-import base64
 import tblib.pickling_support
 import xml.etree.ElementTree
 
@@ -37,6 +35,10 @@ except ImportError:
 import dask
 from dask import istask
 
+# Import config serialization functions here for backward compatibility
+from dask.config import serialize as serialize_for_cli  # noqa
+from dask.config import deserialize as deserialize_for_cli  # noqa
+
 # provide format_bytes here for backwards compatibility
 from dask.utils import (  # noqa
     format_bytes,
@@ -55,7 +57,7 @@ try:
 except ImportError:
     PollIOLoop = None  # dropped in tornado 6.0
 
-from .compatibility import PYPY, WINDOWS, get_running_loop
+from .compatibility import PYPY, WINDOWS
 from .metrics import time
 
 
@@ -64,6 +66,12 @@ try:
 except ImportError:
     thread_state = threading.local()
 
+# For some reason this is required in python >= 3.9
+if WINDOWS:
+    import multiprocessing.popen_spawn_win32
+else:
+    import multiprocessing.popen_spawn_posix
+
 logger = _logger = logging.getLogger(__name__)
 
 
@@ -293,6 +301,7 @@ def sync(loop, func, *args, callback_timeout=None, **kwargs):
     """
     Run coroutine in loop running in separate thread.
     """
+    callback_timeout = parse_timedelta(callback_timeout, "s")
     # Tornado's PollIOLoop doesn't raise when using closed, do it ourselves
     if PollIOLoop and (
         (isinstance(loop, PollIOLoop) and getattr(loop, "_closing", False))
@@ -312,11 +321,15 @@ def sync(loop, func, *args, callback_timeout=None, **kwargs):
 
     @gen.coroutine
     def f():
+        # We flag the thread state asynchronous, which will make sync() call
+        # within `func` use async semantic. In order to support concurrent
+        # calls to sync(), `asynchronous` is used as a ref counter.
+        thread_state.asynchronous = getattr(thread_state, "asynchronous", 0)
+        thread_state.asynchronous += 1
         try:
             if main_tid == threading.get_ident():
                 raise RuntimeError("sync() called from thread of running loop")
             yield gen.moment
-            thread_state.asynchronous = True
             future = func(*args, **kwargs)
             if callback_timeout is not None:
                 future = asyncio.wait_for(future, callback_timeout)
@@ -324,7 +337,8 @@ def sync(loop, func, *args, callback_timeout=None, **kwargs):
         except Exception as exc:
             error[0] = sys.exc_info()
         finally:
-            thread_state.asynchronous = False
+            assert thread_state.asynchronous > 0
+            thread_state.asynchronous -= 1
             e.set()
 
     loop.add_callback(f)
@@ -371,10 +385,8 @@ class LoopRunner:
                 # We're expecting the loop to run in another thread,
                 # avoid re-using this thread's assigned loop
                 self._loop = IOLoop()
-            self._should_close_loop = True
         else:
             self._loop = loop
-            self._should_close_loop = False
         self._asynchronous = asynchronous
         self._loop_thread = None
         self._started = False
@@ -740,26 +752,6 @@ def truncate_exception(e, n=10000):
         return e
 
 
-def tokey(o):
-    """Convert an object to a string.
-
-    Examples
-    --------
-
-    >>> tokey(b'x')
-    b'x'
-    >>> tokey('x')
-    'x'
-    >>> tokey(1)
-    '1'
-    """
-    typ = type(o)
-    if typ is str or typ is bytes:
-        return o
-    else:
-        return str(o)
-
-
 def validate_key(k):
     """Validate a key as received on a stream."""
     typ = type(k)
@@ -778,25 +770,6 @@ def _maybe_complex(task):
     )
 
 
-def convert(task, dsk, extra_values):
-    if type(task) is list:
-        return [convert(v, dsk, extra_values) for v in task]
-    if type(task) is dict:
-        return {k: convert(v, dsk, extra_values) for k, v in task.items()}
-    if istask(task):
-        return (task[0],) + tuple(convert(x, dsk, extra_values) for x in task[1:])
-    try:
-        if task in dsk or task in extra_values:
-            return tokey(task)
-    except TypeError:
-        pass
-    return task
-
-
-def str_graph(dsk, extra_values=()):
-    return {tokey(k): convert(v, dsk, extra_values) for k, v in dsk.items()}
-
-
 def seek_delimiter(file, delimiter, blocksize):
     """Seek current file to next byte after a delimiter bytestring
 
@@ -924,7 +897,6 @@ def ensure_bytes(s):
 
     Examples
     --------
-
     >>> ensure_bytes('123')
     b'123'
     >>> ensure_bytes(b'123')
@@ -945,7 +917,6 @@ def ensure_bytes(s):
 
 def divide_n_among_bins(n, bins):
     """
-
     >>> divide_n_among_bins(12, [1, 1])
     [6, 6]
     >>> divide_n_among_bins(12, [1, 2])
@@ -972,32 +943,6 @@ def mean(seq):
     return sum(seq) / len(seq)
 
 
-if hasattr(sys, "is_finalizing"):
-
-    def shutting_down(is_finalizing=sys.is_finalizing):
-        return is_finalizing()
-
-
-else:
-    _shutting_down = [False]
-
-    def _at_shutdown(l=_shutting_down):
-        l[0] = True
-
-    def shutting_down(l=_shutting_down):
-        return l[0]
-
-    atexit.register(_at_shutdown)
-
-
-shutting_down.__doc__ = """
-    Whether the interpreter is currently shutting down.
-    For use in finalizers, __del__ methods, and similar; it is advised
-    to early bind this function rather than look it up when calling it,
-    since at shutdown module globals may be cleared.
-    """
-
-
 def open_port(host=""):
     """Return a probably-open port
 
@@ -1196,7 +1141,7 @@ if not is_server_extension:
 
     if is_kernel():
         try:
-            get_running_loop()
+            asyncio.get_running_loop()
         except RuntimeError:
             is_kernel_and_no_running_loop = True
 
@@ -1286,6 +1231,7 @@ def color_of(x, palette=palette):
     return palette[n % len(palette)]
 
 
+@functools.lru_cache(None)
 def iscoroutinefunction(f):
     return inspect.iscoroutinefunction(f) or gen.is_coroutine_function(f)
 
@@ -1383,8 +1329,7 @@ def parse_ports(port):
     return ports
 
 
-def is_coroutine_function(f):
-    return asyncio.iscoroutinefunction(f) or gen.is_coroutine_function(f)
+is_coroutine_function = iscoroutinefunction
 
 
 class Log(str):
@@ -1415,11 +1360,11 @@ def cli_keywords(d: dict, cls=None, cmd=None):
 
     Parameters
     ----------
-    d: dict
+    d : dict
         The keywords to convert
-    cls: callable
+    cls : callable
         The callable that consumes these terms to check them for validity
-    cmd: string or object
+    cmd : string or object
         A string with the name of a module, or the module containing a
         click-generated command with a "main" function, or the function itself.
         It may be used to parse a module's custom arguments (i.e., arguments that
@@ -1469,13 +1414,7 @@ def is_valid_xml(text):
     return xml.etree.ElementTree.fromstring(text) is not None
 
 
-try:
-    _offload_executor = ThreadPoolExecutor(
-        max_workers=1, thread_name_prefix="Dask-Offload"
-    )
-except TypeError:
-    _offload_executor = ThreadPoolExecutor(max_workers=1)
-
+_offload_executor = ThreadPoolExecutor(max_workers=1, thread_name_prefix="Dask-Offload")
 weakref.finalize(_offload_executor, _offload_executor.shutdown)
 
 
@@ -1501,36 +1440,6 @@ async def offload(fn, *args, **kwargs):
     return await loop.run_in_executor(_offload_executor, lambda: fn(*args, **kwargs))
 
 
-def serialize_for_cli(data):
-    """Serialize data into a string that can be passthrough cli
-
-    Parameters
-    ----------
-    data: json-serializable object
-        The data to serialize
-    Returns
-    -------
-    serialized_data: str
-        The serialized data as a string
-    """
-    return base64.urlsafe_b64encode(json.dumps(data).encode()).decode()
-
-
-def deserialize_for_cli(data):
-    """De-serialize data into the original object
-
-    Parameters
-    ----------
-    data: str
-        String serialied by serialize_for_cli()
-    Returns
-    -------
-    deserialized_data: obj
-        The de-serialized data
-    """
-    return json.loads(base64.urlsafe_b64decode(data.encode()).decode())
-
-
 class EmptyContext:
     def __enter__(self):
         pass
@@ -1569,7 +1478,6 @@ class LRU(UserDict):
 
 def clean_dashboard_address(addr, default_listen_ip=""):
     """
-
     Examples
     --------
     >>> clean_dashboard_address(8787)
