@@ -6,15 +6,16 @@ import uuid
 
 from tlz import merge
 
+from dask.utils import stringify
 from .client import Future, Client
-from .utils import tokey, log_errors, TimeoutError, parse_timedelta
-from .worker import get_client
+from .utils import log_errors, TimeoutError, parse_timedelta
+from .worker import get_client, get_worker
 
 logger = logging.getLogger(__name__)
 
 
 class VariableExtension:
-    """An extension for the scheduler to manage queues
+    """An extension for the scheduler to manage Variables
 
     This adds the following routes to the scheduler
 
@@ -88,8 +89,7 @@ class VariableExtension:
 
                 await asyncio.wait_for(_(), timeout=left)
             finally:
-                with suppress(RuntimeError):  # Python 3.6 loses lock on finally clause
-                    self.started.release()
+                self.started.release()
 
         record = self.variables[name]
         if record["type"] == "Future":
@@ -144,8 +144,8 @@ class Variable:
         Name used by other clients and the scheduler to identify the variable.
         If not given, a random name will be generated.
     client: Client (optional)
-        Client used for communication with the scheduler. Defaults to the
-        value of ``Client.current()``.
+        Client used for communication with the scheduler.
+        If not given, the default global client will be used.
 
     Examples
     --------
@@ -164,13 +164,17 @@ class Variable:
     """
 
     def __init__(self, name=None, client=None, maxsize=0):
-        self.client = client or Client.current()
+        try:
+            self.client = client or Client.current()
+        except ValueError:
+            # Initialise new client
+            self.client = get_worker().client
         self.name = name or "variable-" + uuid.uuid4().hex
 
     async def _set(self, value):
         if isinstance(value, Future):
             await self.client.scheduler.variable_set(
-                key=tokey(value.key), name=self.name
+                key=stringify(value.key), name=self.name
             )
         else:
             await self.client.scheduler.variable_set(data=value, name=self.name)
@@ -180,7 +184,7 @@ class Variable:
 
         Parameters
         ----------
-        value: Future or object
+        value : Future or object
             Must be either a Future or a msgpack-encodable value
         """
         return self.client.sync(self._set, value, **kwargs)
@@ -210,7 +214,7 @@ class Variable:
 
         Parameters
         ----------
-        timeout: number or string or timedelta, optional
+        timeout : number or string or timedelta, optional
             Time in seconds to wait before timing out.
             Instead of number of seconds, it is also possible to specify
             a timedelta in string format, e.g. "200ms".
